#ifndef __NAVIGATOR_HH_
#define __NAVIGATOR_HH_
// Navigator.hh
#include "CavePointer.hh"
#include "NavBase.hh"
#include "NavPath.hh"

enum NavMode {GrabModelAbsolute, // grab absolute in model coordinates
              GrabAbsolute,  // grab w/ orientation snapped to wand location
	      GrabModelRelative, // grab w/Relative orientation in model coords
	      GrabRelative, // grab w/ orientation relative to wand
	      Fly,   // fly in the direction the wand is pointing
	             // with velocity defined by joystick_Y
	      Walk,  // same as fly, but with floor-collision detection
	             // so you can follow the floor
	      Off,   // don't move!
	      Reset, // load identity (undo all navigation transformations)
	      Halt,
	      Continue,
	      Path	 // Use Mike Stephens path Interpolator 
	      		 // whereby the viewer follows a predefined "track"
	      		 // position on the track is controlled by the joystick
              };

//==========>Navigator<===============================
/*----------------------------------------------------
Class: Navigator
Purpose: A sample modal interface using the above 
         navigation classes.  This class is suitable
	 for use within a menuing system.
Sample Usage#1: (from main_event_loop of pfnav.C example)
   Navigator navigation(nav_dcs); // Initialize the Navigator
  // will eventually be Navigator navigation(nav_dcs,floor);
  // where floor is the bathymetry geometry
  while (!CAVEgetbutton(CAVE_ESCKEY)) {
    pfSync();
    pfCAVEPreFrame();
    pfFrame();
    pfCAVEPostFrame();
    if(CAVEBUTTON1)
       navigation.update(GrabRelative);  // grab object while button1 down
    else
       navigation.update(Fly); // Otherwise, fly
    if(CAVEBUTTON3)         // if cavebutton#3 is pressed, then
      navigation.update(Reset); // reset to initial position
  }

Sample Usage#2:
  // IN SOME OTHER THREAD OR PART OF THE CODE
  // THE USER INTERFACE IS USED TO SELECT NAVIGATION MODES
  switch(menu_choice){
     case FLY:
          navigation.setMode(Fly);
     case WALKTHRU:
          navigation.setMode(Walk);
     case GRABSCENE:
          navigation.setMode(GrabRelative);
     default:
          navigation.setMode(Off);
  }
 <<<<<------------------------------------>>>>>
  // MEANWHILE IN THE MAIN_EVENT_LOOP
  while (!CAVEgetbutton(CAVE_ESCKEY)) {
    pfSync();
    pfCAVEPreFrame();
    pfFrame();
    pfCAVEPostFrame();
    navigation.update(); // update whatever the current mode is
  }

------------------------------------------------------*/
class Navigator : public ClassCallback {
private:
  float rSpeed,tSpeed;
  pfDCS *navdcs;
  pfNode *modeldcs; // the dcs part is an anachronism
  NavMode mode;
  NavBase *navstate;
  // pfScene *model;
  int collision_detection,modelnav,navhalt;
public:
  Navigator(pfDCS *nav,pfNode *object);
  Navigator(pfDCS *nav);
  void update(); // use current nav mode (whatever it may be)
  int update(NavMode nav);
  int setMode(NavMode nav);
  inline void setScene(pfNode *m){
    modeldcs=m;
  }
  void collisionDetection(int val);
  void setRotationSpeed(float speed); // where speed is in rad/sec
  void adjustRotationSpeed(float factor); // adjust speed by factor (speed*=factor)
  void setTranslationSpeed(float speed); // where speed is the trans speed in clicks/sec
  void adjustTranslationSpeed(float factor); // adjust speed by factor (speed*=factor)
  virtual int callback(ParamSet *caller,ParamSet *data);
};


#endif

